// -*- C++ -*-
#include "Rivet/Analysis.hh"
#include "Rivet/Particle.fhh"
#include "Rivet/Projections/UnstableFinalState.hh"
#include "Rivet/Projections/IdentifiedFinalState.hh"
#include "Rivet/Projections/FinalState.hh"

namespace Rivet {

  class ATLAS_2014_I1282441 : public Analysis {

  public:

    ATLAS_2014_I1282441()
    : Analysis("ATLAS_2014_I1282441")
    {}

  public:

    void init() {

      // Use a large eta range such that we can discriminate on y
      UnstableFinalState UFS(-10, 10, 0.5*GeV);
      IdentifiedFinalState PHI(UFS);
      PHI.acceptIdPair(PID::PHI); //PID::PHI
      addProjection(PHI, "PHI_PROJECTION");

      //FinalState FS(-2, 2, 230*MeV);
      IdentifiedFinalState KPLUSMINUS(-2.0, 2.0, 230*MeV);
      KPLUSMINUS.acceptIdPair(PID::KPLUS); //PID::K
      addProjection(KPLUSMINUS, "KPLUSMINUS_PROJECTION");

      nPassed = 0;
      h_phi_rapidity = bookHisto1D(1,1,1); 
      h_phi_pT       = bookHisto1D(2,1,1); 
      
    }

    void analyze(const Event& event) {

      const double weight = event.weight();

      const Particles& ks = applyProjection<IdentifiedFinalState>(event, "KPLUSMINUS_PROJECTION").particles();
      Particles _kp;
      Particles _km;
      foreach( const Particle& p, ks ) {
        if ( p.hasAncestor(333) == false)  { MSG_DEBUG("-- K not from phi."); continue; }
        if ( fabs(p.momentum().vector3().mod()) > 800*MeV ) { MSG_DEBUG("-- p K too high."); continue; }
        if ( p.pdgId() > 0 ) {
          _kp.push_back(p);
        } else {
          _km.push_back(p);
        }
      }

      const ParticleVector& phis = applyProjection<FinalState>(event, "PHI_PROJECTION").particles();
      Particles _p;
      foreach( const Particle& p, phis ) {
        if ( p.absrap() > 0.8 ) { MSG_DEBUG("-- Y phi too high."); continue; }
        if ( p.pT() > 1.2*GeV ) { MSG_DEBUG("-- pT phi too high."); continue; }
        _p.push_back(p);
      }

      // Find Phi -> KK decays through matching of the kinematics
      if (_kp.size() >= 1 && _km.size() >= 1 && _p.size() >= 1) {
        MSG_DEBUG("---- THIS EVENT: WE HAVE THIS MANY PHI:" << _p.size() << ", K+:" << _kp.size() << ", K-:" << _km.size());
        for (unsigned int P=0; P < _p.size(); ++P) {
          for (unsigned int KM=0; KM < _km.size(); ++KM) {
            for (unsigned int KP=0; KP < _kp.size(); ++KP) {
              FourMomentum _mom = ( _kp.at(KP).momentum() + _km.at(KM).momentum() );
              if ( isZero( _mom.mass() - _p.at(P).mass(), 1e-5 ) == true ) {
                MSG_DEBUG("------ ACCEPTED COMBINATORIC: PHI#:" << P << " K+#:" << KP << " K-#:" << KM);
                h_phi_rapidity->fill( fabs(_p.at(P).momentum().rapidity()), weight );
                h_phi_pT      ->fill( fabs(_p.at(P).momentum().pT() / MeV), weight );
                nPassed+=1;
              } else {
                MSG_DEBUG("------ REJECTED COMBINATORIC: PHI#:" << P << " K+#:" << KP << " K-#:" << KM << " MASS DIFFERENCE IS " << _mom.mass() - _p.at(P).mass());
              }
            }
          }
        }
      }

    }

    void finalize() {
      scale(h_phi_rapidity, (crossSection() / microbarn)/sumOfWeights());
      scale(h_phi_pT,       (crossSection() / microbarn)/sumOfWeights());
    }

  private:

    int nPassed;
    Histo1DPtr h_phi_rapidity;
    Histo1DPtr h_phi_pT;

  };

  DECLARE_RIVET_PLUGIN(ATLAS_2014_I1282441);

}
